#!/usr/bin/python
"""Convert .klc file to JavaScript for keyboard.js script
"""
import os.path, sys, unicodedata

def convert(klc):
    klc= unicode(klc, 'utf-16').replace('\r\n', '\n')
    lines= [line.split('//', 1)[0].strip() for line in klc.split('\n')]
    lines= [[part for part in line.split('\t') if part!=''] for line in lines if line!='']

    def getsection(*match):
        for ix0 in range(len(lines)):
            if tuple(lines[ix0][:len(match)])==match:
                break
        else:
            return []
        for ix1 in range(ix0+1, len(lines)):
            try:
                int(lines[ix1][0], 16)
            except ValueError:
                ix1-= 1
                break
        return lines[ix0:ix1+1]

    def readchar(s):
        dead= s.endswith('@') and len(s)>=2
        if dead:
            s= s[:-1]
        if len(s)==1:
            return s, dead
        if s=='-1':
            return '\n', False
        return unichr(int(s, 16)), dead

    layout= {'': {}}
    deadchars= []

    for line in getsection('LAYOUT')[1:]:
        key, chars= line[1], (line[3], line[4], line[6], line[7])
        if key=='DECIMAL': continue
        chs, deads= [], []
        for char in chars:
            ch, dead= readchar(char)
            if dead:
                deadchars.append(ch)
            chs.append(ch)
            deads.append(' @'[dead])
        layout[''][key]= ''.join(chs+deads)

    for firstchar in deadchars:
        lookup= {}
        for line in getsection('DEADKEY', '%04x' % ord(firstchar))[1:]:
            secondchar, _= readchar(line[0])
            resultchar, _= readchar(line[1])
            lookup[secondchar]= resultchar

        if firstchar not in layout:
            layout[firstchar]= {}
        for key in layout['']:
            chars= list(layout[''][key][:4])
            chars= ''.join([lookup.get(char, '\n') for char in chars])
            if chars!='\n\n\n\n':
                layout[firstchar][key]= chars+'    '

    charnames= {}
    for lookup in layout.values():
        for chs in lookup.values():
            for ch in chs[:4]:
                if ch!='\n' and ch not in charnames:
                    try:
                        charnames[ch]= unicodedata.name(ch).title()
                    except ValueError:
                        sys.stderr.write('Warning: Python knows no name for character %r\n' % ch)

    return 'keyboard_LAYOUT= %s\n\nkeyboard_CHARNAMES= %s' % (prettystr(layout), prettystr(charnames))


def prettystr(o):
    if isinstance(o, unicode):
        return repr(o)[1:]
    if isinstance(o, str):
        return repr(o)
    if isinstance(o, dict):
        contents= []
        for key, value in sorted(o.items()):
            contents.append('    %s: %s' % (prettystr(key), prettystr(value).replace('\n', '\n    ')))
        return '{\n%s\n}' % (',\n'.join(contents))
    return 'null'


if __name__=='__main__':
    if len(sys.argv)>=2:
        inpath= sys.argv[1]
    else:
        sys.stderr.write('Syntax: convert.py <input.klc> [<output.js>]\n')
        sys.exit(-1)
    if len(sys.argv)>=3:
        outpath= sys.argv[2]
    else:
        leaf= os.path.basename(inpath)
        outpath= os.path.join(os.path.dirname(inpath), os.path.splitext(leaf)[0]+'.js')
    klc= open(inpath, 'rb').read()
    js= convert(klc)
    open(outpath, 'wb').write(js)
